# land_mine.py

from draftsman.classes.entity import Entity
from draftsman.classes.vector import Vector, PrimitiveVector
from draftsman.constants import ValidationMode
from draftsman.utils import get_first

from draftsman.data.entities import land_mines

from typing import Any, Literal, Optional, Union
from pydantic import ConfigDict


class LandMine(Entity):
    """
    An entity that explodes when in proximity to another force.
    """

    class Format(Entity.Format):
        model_config = ConfigDict(title="LandMine")

    def __init__(
        self,
        name: Optional[str] = get_first(land_mines),
        position: Union[Vector, PrimitiveVector] = None,
        tile_position: Union[Vector, PrimitiveVector] = (0, 0),
        tags: dict[str, Any] = {},
        validate_assignment: Union[
            ValidationMode, Literal["none", "minimum", "strict", "pedantic"]
        ] = ValidationMode.STRICT,
        **kwargs
    ):
        """
        TODO
        """

        super().__init__(
            name,
            land_mines,
            position=position,
            tile_position=tile_position,
            tags=tags,
            **kwargs
        )

        self.validate_assignment = validate_assignment

    # =========================================================================

    __hash__ = Entity.__hash__
